clipMatrix <- function(img) {
  h <- dim(img)[1]
  w <- dim(img)[2]

  for(y in 1:h) {
    for(x in 1:w) {
      img[y,x] <- max(0, min(1, img[y,x]))
    }
  }

  img
}

tm.addNoise <- function(img,
                        noiseType = "normal",
                        scale     = 0.1,
                        clipRange = c(0,1),
                        percent   = NULL) {

  data <- img@data
  h    <- dim(data)[1]
  w    <- dim(data)[2]
  n    <- w*h

  
  if(noiseType == "normalm") {
    noise <- array(rnorm(n,mean=1,sd=scale), dim=c(h,w))
  } else if (noiseType == "normal") {
    noise <- array(rnorm(n,sd=scale), dim=c(h,w))
  } else if (noiseType == "saltpepper") {
    noise <- array(runif(n,min=0,max=scale), dim=c(h,w))
    noise[noise < scale/2]  <- 0
    noise[noise >= scale/2] <- scale
  } else if (noiseType == "photon") {
      noise <- array(0, dim=c(h,w))
      for(y in 1:h) {
        for(x in 1:w) {
          noise[y,x] <-
            tm.photonNoise(data[y,x],
                           gammaCorrection = 1,
                           maxValue        = 1,
                           fullWell        = 48800,
                           fullWellIso     = 100,
                           iso             = scale)
        }
      }
    } else {
      noise <- array(runif(n,min=-scale,max=scale),
                     dim=c(h,w))
    }

  if(is.null(percent)) {
    if(noiseType == "normalm") {
      img@data <- data + noise
    } else if((noiseType == "saltpepper") ||
              (noiseType == "photon")) {
      img@data <- noise
    } else {
      img@data <- data + noise
    }
  } else {
    mask             <- array(runif(n,min=0,max=1), dim=c(h,w))<percent
    if(noiseType == "normalm") {
      data[mask] <- (data * noise)[mask]
    } else if((noiseType == "saltpepper") ||
              (noiseType == "photon")) {
      data[mask] <- noise[mask]
    } else {
      data[mask] <- (data + noise)[mask]
    }
    img@data     <- data
  }
  
  if(!is.null(clipRange)) {
    ia.clipRange(img, clipRange = clipRange)
  } else {
    img
  }
}
