#
# REMARK: the imagematrix returned by rimage is a 2 (or 3) dimensional array
# if a 3D array last dim is referrng to color channels so
# img[,,1] returns the first channel
# img[,,2] returns the second channel
# as.animage()img[,,1] create the corresponding animage
#
tm.rotationMatrix <- function(theta)
  {
    mat <- array(0, c(2,2));

    mat[1,1] <-  cos(theta);
    mat[1,2] <- -sin(theta);
    mat[2,1] <-  sin(theta);
    mat[2,2] <-  cos(theta);

    mat
  }

tm.scalingMatrix <- function(scalex, scaley)
  {
    if(missing(scaley)) {
      scaley = scalex;
    }
    
    mat <- array(c(scalex, 0, 0, scaley), c(2,2));

    mat
  }

tm.affineMatrix <- function(theta, scalex, scaley, translation = c(0,0))
  {
    rMat <- tm.rotationMatrix(theta)
    sMat <- tm.scalingMatrix(scalex, scaley)
    lMat <- sMat %*% rMat

    
    aMat <- array(0, c(3,3));

    for(j in 1:2) {
      for(i in 1:2) {
        aMat[j,i] <- lMat[j,i]
      }
    }

    aMat[1,3] <- translation[1]
    aMat[2,3] <- translation[2]

    aMat
  }

#
# Perform image smoothing via separable Lanczos sampling: this is supposed
# to be the first step in an image resampling operation. If factor
# is less than 1 the argument image is returned unaltered
#
tm.lanczosSmoothing <- function(img, factor = 2.0, support = 2, filler = FALSE)
  {
    if(factor > 1.0) {
      actualSupport <- as.integer(support * factor + 0.5)
      kernel        <- array(0, c(1, 2*actualSupport+1))

      
      i = 1;
      for(x in -actualSupport:actualSupport) {
        kernel[1, i] <- tm.Lanczos(x/factor,support)
        i <- i+1
      }

      kernel         <- kernel / sum(kernel)

      if(filler) {
        filler         <- ia.add(ia.mult(img, 0.0), 1.0)
        filler@outside <- 0
        
        ia.div(ia.convolution(ia.div(ia.convolution(img, kernel),
                                     ia.convolution(filler, kernel)),
                              t(kernel)),
               ia.convolution(filler, t(kernel)))
      } else {
        ia.convolution(ia.convolution(img, kernel), t(kernel))
      }
    } else {
      img
    }
  }
 
tm.normalizeFaceImage <- function(image, xleft, yleft, xright, yright, ee = 55, smooth = TRUE)
  {
    theta  <- atan2(yright-yleft, xright-xleft)
    center <- c(-(xright+xleft)*0.5, -(yright+yleft)*0.5)
    scale  <- ee / sqrt((xright-xleft)**2+(yright-yleft)**2)
    aMat   <- tm.affineMatrix(-theta, scale, scale, center)

    if(smooth == FALSE)
      ia.affine(image, aMat)
    else
      ia.affine(tm.lanczosSmoothing(image, 1.0 / scale, filler = TRUE), aMat)
  }
