#
# \litcode{Digital camera photon noise}
#
# Quantum nature of light manifests itself as a Poisson noise
# affecting digital imaging. In order to simulate this noise process
# correctly we need to now a few parameters: the maximum number of electrons
# that fit within a pixel well and the corresponding ISO sensitivity,
# the ISO sensitivity at which the picture is taken, the maximum intensity
# value, and the gamma correction factor if any. These data allow us to
# map an intensity value in the digital image into and absolute number of electrons
# which is proportional to the number of photons. The latter provides
# the (single) parameter $\lambda$ controlling the Poisson process.
# The default \code{fullWell} value is typical of a high-end digital
# reflex camera
#
tm.photonNoise <- function(x,
                           maxValue        =   255,
                           gammaCorrection =   (1.0 / 2.2),
                           fullWell        = 51400,
                           fullWellIso     =   100,
                           iso             =   200)
  {
    #
    # The first step is mapping the intensity value
    # into a number proportional to the number of
    # photons:
    #
    lambdaOrig      <- ((x/maxValue)**(1.0/gammaCorrection)
                        * fullWell
                        * (fullWellIso / iso))
    #
    # A new, noisy value is generated according to
    # the corresponding Poisson distribution:
    #
    lambdaPerturbed <- rpois(1, lambdaOrig)
    #
    # and it is mapped back the digital image context
    #
    return(maxValue
           * ((lambdaPerturbed*(iso/fullWellIso))/fullWell)**gammaCorrection)
    #
  }
