\name{tm.sRGB}

\alias{tm.srgbToLinear}
\alias{tm.linearToSrgb}

\title{sRGB to linear RGB color space mapping}

\description{

  sRGB is a common color space employed in digital imaging: mappings to
  and from the linear RGB space is required.

}

\usage{
tm.srgbToLinear(x)
tm.linearToSrgb(X)
}

\arguments{
  \item{x}{a real in the interval [0,1] representing an sRGB color component}
  \item{X}{a real in the interval [0,1] representing a linear color component}
}

\value{
  
  The functions return a real value in the interval [0,1].
  
}

\details{

 While perceptively appropriate, the sRGB color space does not correspond linearly to the
 actual number of photons. As the simulation of some physically
 accurate sensor characteristics requires the linear RGB values, two
 mapping functions are provided.

}

\author{Roberto Brunelli \email{brunelli@fbk.eu}}

\examples{
#
# Get sample images
#
tiffdir          <- file.path(system.file(package = "TeMa"),
                              "sampleimages/cameraSimulation")
#
i_persp          <- ia.scale(ia.readAnimage(file.path(tiffdir,
                                            "persp_iso12233_large.tif")),
                             maxValue = 255)
i_persp_linear   <- lapply(i_persp,
                           function(img) {ia.apply(img,
                                                   function(x, y, z) {tm.srgbToLinear(z)})})

}

\keyword{array}
