/*
 * (c) 1993,1994,1995,1996 by ITC-IRST
 *
 */

/*
  
  Claudio Andreatta:
  1. 
  Added operator && for vector concatenation
  
  2. Added:
  write as Array rows

  void                  write(ostream&s);
  void                  write(char *filename);
  void                  write(char *filename, int fmt);
  void                  write(ostream&s, int fmt);

*/


#ifndef	_vector_h
#define _vector_h 1

#include	<cstdlib>
#include	<cmath>
#include	<iostream>

#if !defined(_WIN32)
// #include    <stream.h>
#include    <values.h>
#else
#include    <streamb>
#endif

#include	<fstream>
#include	<arrayTools/global.h>
#include	<arrayTools/Endian.h>
#include	<string>
#include        <cstdio>
#include        <arrayTools/io_heads.h>

#include <arrayTools/namespace_header.h>


namespace _ARRAYTOOLS_NAMESPACE
{
  using namespace std;


  using std::sqrt; 

extern int	created_vectors;
extern int	deleted_vectors;
extern int      access_control;
extern void     VectorError();
extern void     VectorError(char *message);

void VectorCount(char* label);


 template<class T> class vector;
template<class T> double     Norm(vector<T>&);
template<class T> void       DeleteIfTemporary(vector<T>  &v);
template<class T> ostream&   operator<<(ostream&s, vector<T> &v);

template<class T> class vector{
  T	        *elts;
  char		reuse;
  int		first_idx;
  int		last_idx;
  int		size;
  int           id;
 private:
  inline void		dispose(int flag){ reuse = (flag == 0) ? FALSE : TRUE;}
  void			FreeOwnStorage();
  void			TransferStorage(int*, int*, int*, T**);
 public:
  vector();
  vector(unsigned int);
  vector(int, int);
  vector(vector<T> &); 
  vector(const vector<T> &);
  ~vector();

  void        reset()               { (*this) = (new vector<T>)->temporary(); }
  void        reset(int n)          { (*this) = (new vector<T>(n))->temporary(); }
  void        reset(int c0, int c1) { (*this) = (new vector<T>(c0,c1))->temporary(); }
  template<class U> void reset(const vector<U> & V) { (*this).reset(V.first_index(), V.last_index()); }


  inline int		dispose() { return((int) reuse);}

  vector<T>&		translate_at(int);
  vector<T>&		temporary();
  vector<T>&		operator=(vector<T>&);
  vector<T>&		operator=(const vector<T>&);
  vector<T>&		operator=(T value);
  vector<T>&		versor();

  vector<T>&            operator&&(vector<T>&);
  vector<T>&	        operator-();
  vector<T>&	        operator-(vector<T>&);
  vector<T>&	        operator+(vector<T>&);
  vector<T>&	        operator&(vector<T>&);
  vector<T>&	        operator%(vector<T>&);
  vector<T>&	        operator+=(double);
  vector<T>&	        operator*=(double);
  double	        operator*(vector<T>&);
  boolean	        operator==(vector<T>&);
  boolean	        operator!=(vector<T>&);
  void                  writeAscii(char    *name);
  void                  writeAscii(ostream &s);
  void                  writeAscii(char    *name, char mode);
  void                  writeAscii(ostream &s,    char mode);
  void                  write(ostream&s);
  void                  write(char *filename);
  void                  write(char *filename, int fmt);
  void                  write(ostream&s, int fmt);

  vector<T>&	        operator*(double k);
//!  friend vector<T>&	operator*<>(double k, vector<T>& v);
#if !defined(_WIN32)
  friend double		Norm<>(vector<T>&);
  friend void		DeleteIfTemporary<>(vector<T>  &v);
  friend ostream&	operator<<<>(ostream&s, vector<T> &v);
#else
  friend double     Norm(vector<T>&);
  friend void       DeleteIfTemporary(vector<T>  &v);
  friend ostream&   operator<<(ostream&s, vector<T> &v);
#endif

  inline int		first_index() const	{ return(first_idx);}
  inline int		last_index() const 	{ return(last_idx);}
  inline int		ID() const 	        { return(id);}
  inline unsigned int	length() const  	{ return(size);}
  inline T*	        storage()       { return(elts);}
  inline T*	        chunk()         { return(elts+first_idx);}
  inline T&	        operator[](int);
};


//! Free own data storage, and reset internal  data
//!

template<class T> void vector<T>::FreeOwnStorage()
 {
   if(elts != NULL)
     delete[] (elts + first_idx);

   elts 	=  NULL;
   size		=  0;
   first_idx	=  0;
   last_idx	= -1;
 }
  

//! WARNING: this function must be used with extreme care.
//! It returns via a set of pointers all its internal 
//! structure, and reset itself to an empty stab.
//! It is used to improve assignment efficiency,
//! by transferring data of a temporary vector to the 
//! LHS of an assignment statement.
//!

template<class T> void vector<T>::TransferStorage(int	*ret_first_idx,
						  int	*ret_size,
						  int	*ret_last_idx,
						  T	**ret_elts)
			       
 {
   *ret_elts 		= elts;
   *ret_first_idx	= first_idx;
   *ret_size		= size;
   *ret_last_idx	= last_idx;

   elts 	= NULL;
   first_idx   	= 0;
   last_idx     = -1;
   size		= 0;
 }
    

//! Create a vector with no associated elements 
//!

template<class T>  vector<T>::vector()
 {
   created_vectors++;

   id           = created_vectors;
   elts		= NULL;
   reuse	= FALSE;
   first_idx	= 0;
   last_idx	= -1;
   size		= 0;
 }


//! Create a vector of <n> elements
//!	indexes start from  zero
//!

template<class T>  vector<T>::vector(unsigned int n)
 {
   int          vectorBytes = n * sizeof(T);
   
   created_vectors++;
   id           = created_vectors;

   elts		= (T *)(new memUnit[(n*sizeof(T)+MEMUNITSIZE-1)/MEMUNITSIZE]);
   reuse	= FALSE;
   size		= n;
   first_idx	= 0;
   last_idx	= first_idx + size - 1;

   memset(elts, 0, vectorBytes);
 }


//! Create a vector of <l_idx - f_idx + 1> elements
//!	legal indexes are from  <f_idx> to <l_idx>
//! 	inclusive
//!

template<class T>  vector<T>::vector(int f_idx, int l_idx)
 {
   int          vectorBytes;
   
   created_vectors++;
   id           = created_vectors;
   
   if(f_idx > l_idx)
      {
	cerr << "Illegal range for vector index!\n";
	exit(-1);
      }
   
   reuse	= FALSE;
   first_idx	= f_idx;
   last_idx	= l_idx;
   size		= last_idx - first_idx  + 1;
   vectorBytes  = size * sizeof(T);
   elts		= (T *)(new memUnit[(vectorBytes+MEMUNITSIZE-1)/MEMUNITSIZE]);

   memset(elts, 0, vectorBytes);

   elts		-= first_idx;
 }


//! Create a copy of an existing vector. 
//!

template<class T>  vector<T>::vector(vector<T>& v)
 {
   unsigned int	n = v.length();

   created_vectors++;
   id           = created_vectors;
   
   elts		= (T *)(new memUnit[(n*sizeof(T)+MEMUNITSIZE-1)/MEMUNITSIZE]);
   reuse	= FALSE;
   size		= n;
   first_idx	= v.first_index();
   last_idx	= v.last_index();
   elts		-= first_idx;

   memcpy(&(elts[first_idx]), &(v[first_idx]), sizeof(T)*n);

   DeleteIfTemporary(v);
 }

//! Create a copy of an existing vector. 
//!

template<class T>  vector<T>::vector(const vector<T>& v)
 {
   //cerr << "vector<T>::vector(const vector<T>& v)" << endl;

   unsigned int	n = v.length();

   created_vectors++;
   id           = created_vectors;
   
   elts		= (T *)(new memUnit[(n*sizeof(T)+MEMUNITSIZE-1)/MEMUNITSIZE]);
   reuse	= FALSE;
   size		= n;
   first_idx	= v.first_index();
   last_idx	= v.last_index();
   elts		-= first_idx;

   memcpy(&(elts[first_idx]), &(v.elts[first_idx]), sizeof(T)*n);
 }



//! Delete a vector by freeing its storage area
//!

template<class T>  vector<T>::~vector()
 {
   if(size == -1)
      {
	cerr << "Internal error: trying to delete already deleted vector!\n";
	cerr << "                Improper use of temporary vector?       \n";

	VectorError();
	
	exit(-1);
      }

   if(elts != NULL)
     delete[] (elts + first_idx);

   elts		= NULL;
   reuse	= FALSE;
   size		= -1;
   first_idx	= 0;
   last_idx	= -1;
   
   deleted_vectors ++;
 }


//! Reference operator for accessing the vector elements
//! If the package is compiled with the flag CHECKS undefined, 
//! no checking is done on the access index.
//!

template<class T> T&	vector<T>::operator[](int i)
 {
#ifndef CHECKS 
	return(elts[i]);
#else
      {
	if(size == -1)
	   {
	     cerr << "Illegal reference to elements of a delete vector!\n";
	     cerr << "         Improper use of temporary vector?        \n";
	     exit(-1);
	   }
	else if((i < first_idx) || (i > last_idx))
	   {
	     char err_msg[256];
	     
	     sprintf(err_msg, "Vector: index out of range: %d not in [%d,%d]",
		     i, first_idx, last_idx);
	     
	     VectorError(err_msg);

	     exit(-1);
	   }

	return(elts[i]);
      }
#endif
 }


//! Delete a vector if it was tagged for deletion
//!

template<class T> void DeleteIfTemporary(vector<T>& v)
 {
   if(v.dispose() == TRUE)
     delete &v;
 }
    

//! Assignment operator.  Data of right hand side are copied into 
//! the data region of the left hand side.
//! If the RHS is tagged for deletion, it is deleted.
//! The LHS is protected from deletion
//!

template<class T> vector<T>&	vector<T>::operator=(vector<T>& v)
 {
   int	        n = v.length();

   if(v.dispose() ==  TRUE)
      {
	//! If v is temporary, exploit its data
	//! structure

	(*this).FreeOwnStorage();
	v.TransferStorage(&first_idx, &size, &last_idx, &elts);
	reuse	= FALSE;
	delete &v;
      }
   else
      {
	if(n > 0)
	   {
	     elts	= elts + first_idx;
	     if((elts != NULL) && (n != size))
		{
		  delete[] 	elts;
		  elts		= (T *)(new memUnit[(n*sizeof(T)+MEMUNITSIZE-1)/MEMUNITSIZE]);
		}
	     else if (elts == NULL) 
		{
		  elts		= (T *)(new memUnit[(n*sizeof(T)+MEMUNITSIZE-1)/MEMUNITSIZE]);
		}

	     size		= n;
	     reuse		= FALSE;
	     first_idx	 	= v.first_index();
	     last_idx		= first_idx + size - 1;
	     elts	 	-= first_idx;

	     memcpy(&(elts[first_idx]), &(v[first_idx]), sizeof(T)*n);
	   }
	else if(elts != NULL)
	   {
	     delete[] (elts + first_idx);
	     elts	= NULL;
	     reuse	= FALSE;
	     first_idx  = 0;
	     size	= 0;
	     last_idx   = -1;
	   }
	  
	DeleteIfTemporary(v);
      }
   
   return(*this);
 }

//! Assignment operator with const argument
//!

template<class T> vector<T>&	vector<T>::operator=(const vector<T>& v)
 {
   //cerr << "vector<T>::operator=(const vector<T>& A)\n";

   int	        n = v.length();

   if(n > 0)
     {
       elts	= elts + first_idx;
       if((elts != NULL) && (n != size))
	 {
	   delete[] 	elts;
	   elts		= (T *)(new memUnit[(n*sizeof(T)+MEMUNITSIZE-1)/MEMUNITSIZE]);
	 }
       else if (elts == NULL) 
	 {
	   elts		= (T *)(new memUnit[(n*sizeof(T)+MEMUNITSIZE-1)/MEMUNITSIZE]);
	 }

       size		= n;
       reuse		= FALSE;
       first_idx	 	= v.first_index();
       last_idx		= first_idx + size - 1;
       elts	 	-= first_idx;

       memcpy(&(elts[first_idx]), &(v.elts[first_idx]), sizeof(T)*n);
     }
   else if(elts != NULL)
     {
       delete[] (elts + first_idx);
       elts	= NULL;
       reuse	= FALSE;
       first_idx  = 0;
       size	= 0;
       last_idx   = -1;
     }

   return(*this);
 }

template<class T> vector<T>&	vector<T>::operator=(T value)
 {
   int i;
   
   for(i = first_idx; i <= last_idx; i++)
     elts[i] = value;

  return(*this);
 }

//! Return TRUE if the two vectors are equal
//!

template<class T> boolean	vector<T>::operator==(vector<T> &x)
 {
   boolean	b 	= TRUE;
   int		i;
   
   if((first_idx != x.first_index()) || (last_idx != x.last_index()))
     b 	= FALSE;
   else
      {
	for(i = first_idx; (i <= last_idx) && (b == TRUE); i++)
	  b	&= (x[i] == elts[i]) ? TRUE : FALSE;
      }
   

   DeleteIfTemporary(x);
   DeleteIfTemporary(*this);
   
   return(b);
 }


//! Return TRUE if the two vectors differ
//!

template<class T> boolean	vector<T>::operator!=(vector<T> &x)
 {
   return((x==(*this)) ? FALSE : TRUE);
 }


//! Check whether two vectors differ in base index  and/or dimensions.
//! Should they differ, the program exits.  
//!

template<class T> void CheckVectorsConsistency(vector<T> &a, vector<T> &b)
 {
   if((a.first_index() != b.first_index())	||
      (a.length()      != b.length()))
      {
	cerr << "Vectors differ in base indexes and/or dimension!\n";
	exit(-1);
      }
 }


//! Vector concatenation

template<class T> vector<T>& vector<T>::operator&&(vector<T>& a)
{
  int i, j;

  vector<T> *x;
 
 int ai0 = a.first_index();
  int ai1 = a.last_index();
  
  x = new vector<T>(length + a.length());

  i = 0;
  for(j = first_idx; j <= last_idx; j++)
    (*x)[i++] = elts[j];
  for(j = ai0; j <= ai1; j++)
    (*x)[i++] = a[j];

  DeleteIfTemporary(a);
  DeleteIfTemporary(*this);

  return ((*x).temporary());
}


//! Add vector, element by element and return the new resulting vector.
//! If the vectors have different dimensions or base indexes an error 
//! occurs.
//! The result of the operation is tagged for deletion after usage
//!

template<class T> vector<T>&	vector<T>::operator+(vector<T> &a)
 {
   int		i;
   vector<T>	*x;

   CheckVectorsConsistency(a, (*this));

   x	= new vector<T>(first_idx, last_idx);
   
   for(i = first_idx; i <= last_idx; i++)
     (*x)[i] = a[i] + elts[i];

   DeleteIfTemporary(a);
   DeleteIfTemporary(*this);

   return((*x).temporary());
 }


//! Unary minus.
//! The result of the operation is tagged for deletion after usage
//!

template<class T> vector<T>&	vector<T>::operator-()
 {
   int		i;
   vector<T>	*x;

   x	= new vector<T>(first_idx, last_idx);
   
   for(i = first_idx; i <= last_idx; i++)
     (*x)[i] = -elts[i];

   DeleteIfTemporary(*this);
   
   return((*x).temporary());
 }


//! Subtract vector, element by element, and return the new resulting vector.
//! If the vectors have different dimensions or base indexes an error 
//! occurs.
//! The result of the operation is tagged for deletion after usage
//!

template<class T> vector<T>&	vector<T>::operator-(vector<T> &a)
 {
   int		i;
   vector<T>	*x;

   CheckVectorsConsistency(a, *this);

   x	= new vector<T>(first_idx, last_idx);

   for(i = first_idx; i <= last_idx; i++)
     (*x)[i] = elts[i] - a[i];

   DeleteIfTemporary(a);
   DeleteIfTemporary(*this);

   return((*x).temporary());
 }


//! Multiply vector, element by element, and return the new resulting vector.
//! If the vectors have different dimensions or base indexes an error 
//! occurs.
//! The result of the operation is tagged for deletion after usage
//!

template<class T> vector<T>&	vector<T>::operator&(vector<T> &a)
 {
   int		i;
   vector<T>	*x;

   CheckVectorsConsistency(a, *this);

   x	= new vector<T>(first_idx, last_idx);

   for(i = first_idx; i <= last_idx; i++)
     (*x)[i] = elts[i] * a[i];

   DeleteIfTemporary(a);
   DeleteIfTemporary(*this);

   return((*x).temporary());
 }


//! Divide vector, element by element, and return the new resulting vector.
//! If the vectors have different dimensions or base indexes an error 
//! occurs.
//! The result of the operation is tagged for deletion after usage
//!

template<class T> vector<T>&	vector<T>::operator%(vector<T> &a)
 {
   int		i;
   vector<T>	*x;

   CheckVectorsConsistency(a, *this);

   x	= new vector<T>(first_idx, last_idx);

   for(i = first_idx; i <= last_idx; i++)
     (*x)[i] = elts[i] / a[i];

   DeleteIfTemporary(a);
   DeleteIfTemporary(*this);

   return((*x).temporary());
 }

//! In place multiplication by scalar
//!

template<class T> vector<T>&		vector<T>::operator*=(double v)
 {
   int		i;
   T            *_t;
   
   _t   = (*this).storage();
   
   for(i = first_idx; i <= last_idx; i++)
     _t[i] = (T) (_t[i] * v);

   return(*this);
 }

//! In place addition with a  scalar
//!

template<class T> vector<T>&		vector<T>::operator+=(double v)
 {
   int		i;
   T            *_t;
   
   _t   = (*this).storage();
   
   for(i = first_idx; i <= last_idx; i++)
     _t[i] += (T) (v);

   return(*this);
 }


//! Modify base index of self
//!

template<class T> vector<T>& vector<T>::translate_at(int new_base)
 {
   if(elts != NULL)
      {
	elts  		= elts + (first_idx-new_base);
	first_idx	= new_base;
	last_idx	= first_idx + size - 1;
      }

   return(*this);
 }


//! Modify a vector by making it temporary
//!

template<class T> vector<T>& vector<T>::temporary()
 {
   reuse = TRUE;
   
   return(*this);
 }


//! Return a versor (a vector with unit norm) from the given argument
//!

template<class T> vector<T>&	vector<T>::versor()
 {
   vector<T> 	*x;
   double	value = 0.0;
   int		i;

   x = new vector<T>(first_idx, last_idx);

   for(i = first_idx; i <= last_idx; i++)
     value += (*this)[i]*(*this)[i];
   value =  sqrt(value);

   for(i = first_idx; i <= last_idx; i++)
     (*x)[i] = T((*this)[i]/value);
   
   DeleteIfTemporary(*this);
   
   return((*x).temporary());
 }
    

//! Return the dot product of two vectors 
//! If the vectors have different dimensions or base indexes an error 
//! occurs.
//!

template<class T> double	vector<T>::operator*(vector<T> &a) 
 {
   double	dot = 0.0;
   int		i;
   
   CheckVectorsConsistency(*this, a);

   for(i = first_idx; i <= last_idx; i++)
     dot	+= elts[i] * a[i];

   DeleteIfTemporary(a);
   DeleteIfTemporary(*this);
   
   return(dot);
 }
    

//! Multiply a vector by a scalar and return the new resulting vector.
//! The result of the operation is tagged for deletion
//!

template<class T>  vector<T>&	vector<T>::operator*(double k)
 {
   int		i;
   vector<T>	*x;

   x	= new vector<T>(first_idx, last_idx);
   
   for(i = first_idx; i <= last_idx; i++)
     (*x)[i] = elts[i] * k;

   DeleteIfTemporary(*this);

   return((*x).temporary());
 }


template<class T>   vector<T>&	operator*(vector<T>& v, double k)
 {
   return(k*v);
 }


//! Return the norm of a vector 
//!

template<class T> double	Norm(vector<T>& a)
 {
   double	value = 0.0;
   int		first_idx	= a.first_index();
   int		last_idx	= a.last_index();
   int		i;
   
   for(i = first_idx; i <= last_idx; i++)
     value += a[i]*a[i];
   
   DeleteIfTemporary(a);
   
   return(sqrt(value));
 }

    
//! Write vector elements in ASCII format
//!

template<class T> void	vector<T>::writeAscii(ostream &s, char mode)
 {
   int i;
   
   switch(mode)
      {
       case 'r':
	 {
	   for(i = first_idx; i <= last_idx; i++)
	     s << elts[i] << " ";
	   s << "\n";
	 }
	break;
       case 'c':
	 {
	   for(i = first_idx; i <= last_idx; i++)
	     s << elts[i] << "\n";
	 }
	break;
       default:
	fprintf(stderr, "Illegal vector printing mode!\n");
	exit(-1);
      }
 }
   
template<class T> void	vector<T>::writeAscii(ostream &s)
 {
   (*this).writeAscii(s, 'r');
 } 
template<class T> void	vector<T>::writeAscii(char *name, char mode)
 {
   ofstream out(name);
   
   (*this).writeAscii(out, mode);

   out.close();
 }
    
template<class T> void	vector<T>::writeAscii(char *name)
 {
   (*this).writeAscii(name, 'r');
 }


template<class T> void vector<T>::write(ostream&s)
{
  animal_header	header;
  int		i;
  float		value;
  
  header.MAGIC	= lIMAGE;
  header.rows	= 1;
  header.cols	= size;
  header.type	= anFLOAT;
  header.focusx	= first_idx;
  header.focusy	= 0;
  header.outside= 0;
  header.flag	= 0;
  
  fwritex((char *) &header, sizeof(int), sizeof(animal_header)/sizeof(int), s);

  for(i = first_idx; i <= last_idx; i++)
    {
      value	= (*this)[i];
      fwritex((char *) &value, sizeof(float), 1, s);
    }

  DeleteIfTemporary(*this);
}


template<class T> void vector<T>::write(char *filename)
{
  ofstream out(filename);
  
  (*this).write(out);
  
  out.close();
}

template<class T> void vector<T>::write(char *filename, int fmt)
{
  ofstream out(filename);
   
  (*this).write(out, fmt);
   
  out.close();
}

template<class T> void vector<T>::write(ostream&s, int fmt)
{
  animal_header	header;
  int		y;
  
  header.MAGIC	= lIMAGE;
  header.rows	= 1;
  header.cols	= size;
  header.focusx	= first_idx;
  header.focusy	= 0;
  header.outside= 0;
  header.flag	= 0;

  switch(fmt)
     {
      case anCHAR:
	{
	  char value;
	  
	  header.type	= anCHAR;
	  fwritex((char *) &header, sizeof(int), sizeof(animal_header)/sizeof(int), s);


	  for(y = first_idx; y <= last_idx; y++)
	    {
	      value	= (char)((*this)[y]);
	      s.write((char *) &value, sizeof(char));
	    }
	}
       break;
      case anUCHAR:
	{
	  unsigned char value;
	  
	  header.type	= anUCHAR;
	  fwritex((char *) &header, sizeof(int), sizeof(animal_header)/sizeof(int), s);


	  for(y = first_idx; y <= last_idx; y++)
	    {
	      value	= (unsigned char)((*this)[y]);
	      s.write((char *) &value, sizeof(unsigned char));
	    }
	}
       break;
      case anSHORT:
	{
	  int value;
	  
	  header.type	= anSHORT;
	  fwritex((char *) &header, sizeof(short), sizeof(animal_header)/sizeof(int), s);


	  for(y = first_idx; y <= last_idx; y++)
	    {
	      value	= (int)((*this)[y]);
	      fwritex((char *) &value, sizeof(short), 1, s);
	    }
	}
       break;
      case anINT:
	{
	  int value;
	  
	  header.type	= anINT;
	  fwritex((char *) &header, sizeof(int), sizeof(animal_header)/sizeof(int), s);


	  for(y = first_idx; y <= last_idx; y++)
	    {
	      value	= (int)((*this)[y]);
	      fwritex((char *) &value, sizeof(int), 1, s);
	    }
	}
       break;
      case anFLOAT:
	{
	  float value;
	  
	  header.type	= anFLOAT;
	  fwritex((char *) &header, sizeof(int), sizeof(animal_header)/sizeof(int), s);

	  for(y = first_idx; y <= last_idx; y++)
	    {
	      value	= (float)((*this)[y]);
	      fwritex((char *) &value, sizeof(float), 1, s);
	    }
	}
       break;
      case anDOUBLE:
	{
	  double value;
	  
	  header.type	= anDOUBLE;
	  fwritex((char *) &header, sizeof(int), sizeof(animal_header)/sizeof(int), s);

	  for(y = first_idx; y <= last_idx; y++)
	    {
	      value	= (double)((*this)[y]);
	      fwritex((char *) &value, sizeof(double), 1, s);
	    }
	}
       break;
      default:
	{
	  cerr << "Illegal type for formatted vector dump\n";
	  exit(-1);
	}
     }

  DeleteIfTemporary(*this);
}


//! Output vector <v> to stream in the following ASCII format: 
//! (v_1, ... , v_n)
//! Binary I/O is provided within the array class
//!

template<class T> ostream& operator<<(ostream&s, vector<T> &v)
{
  int	first_idx = v.first_index();
  int	last_idx  = v.last_index();
  int	i;

  s << "[" << first_idx << ":" << last_idx << "]";

  s << "." << v.storage() << "."  << v.ID() << ".";;
  
  s << "(";

  s.setf(ios::scientific, ios::floatfield);
  
  s << v[first_idx];
  
  for(i = first_idx + 1; i <= last_idx; i++)
     {
       s << ", " ;

       s << v[i];
     }

  s << ")";

  DeleteIfTemporary(v);

  return(s);
}


//! Cast operator: copy first argument into a vector with the same type
//! of the second argument
//!

template<class T, class U> vector<T>& Cast(vector<U>& b, vector<T>& a)
 {
   int          i, i0, i1;
   vector<T>    *x;
   
   i0 = b.first_index();
   i1 = b.last_index();
   
   x  = new vector<T>(i0, i1);
   
   for(i = i0; i <= i1; i++)
      {
	(*x)[i] = T(b[i]);
      }
   
   DeleteIfTemporary(a);
   DeleteIfTemporary(b);
   
   return((*x).temporary());
 }

}
    
#endif
