#include	<arrayTools/Vector.h>
#include	<arrayTools/Array.h>
#include    <values.h>
#include    <stdio.h>
#include    <unistd.h>

namespace _ARRAYTOOLS_NAMESPACE
{

  //! Round a double to nearest integer
  //!

  int Rint(double value)
  {
    int    sign = 1;
    int    i_value;

    if(value < 0.0)
      {
		value = - value;
		sign  = -1;
      }
   
    i_value = int(value);

    if((value - i_value) > 0.5)
      i_value++;
   
    return(sign * i_value);
  }

  //! Return the affine transform of I according to M and 
  //! translation O
  //!

  array<u_char>&	Affine(array<u_char>& I, array<double>& M, vector<double> &O)
  {
    array<u_char> *R;
    int		 cols, rows;
    int		 X_start, X_end;
    int		 Y_start, Y_end;
    int		 x_start, x_end, x;
    int		 y_start, y_end, y;
    double	 a,  b,  c,  d;
    double	 ia, ib, ic, id;
    double	 x_min, x_max, y_min, y_max;
    double	 xx_min, xx_max, yy_min, yy_max;
    double	 bounds[4][2];
    double	 determinant;
   
   
    //! Read transform data
    //!
    a	= M[0][0];
    b	= M[0][1];
    c	= M[1][0];
    d	= M[1][1];
   
    //! Compute determinant
    //!
    determinant	= a*d - b*c;
   
    //! Check if transform is singular
    //!
    if(determinant == 0)
      {
		cerr << "Singular transform detected by Affine\n";
		exit(-1);
      }
   
    //! Compute inverse transform
    //!
    ia	=   (d / determinant);
    ib	= - (b / determinant);
    ic	= - (c / determinant);
    id	=   (a / determinant);
   

    //! Get bounds of image
    //!
    cols		= I.cols();
    rows		= I.rows();
    X_start      = I.first_col_index();
    X_end        = I.last_col_index();
    Y_start      = I.first_row_index();
    Y_end        = I.last_row_index();
   
    bounds[0][0]	= (double) X_start;
    bounds[0][1]	= (double) Y_start;
    bounds[1][0]	= bounds[0][0] + (double) cols;
    bounds[1][1]	= bounds[0][1]; 
    bounds[2][0]	= bounds[0][0]; 
    bounds[2][1]	= bounds[0][1] + (double) rows;
    bounds[3][0]	= bounds[1][0];
    bounds[3][1]	= bounds[2][1];

    //! Compute new bounding box
    //!
    x_min = x_max	= a * (bounds[0][0] + O[0]) + b * (bounds[0][1] + O[1]);
    y_min = y_max	= c * (bounds[0][0] + O[0]) + d * (bounds[0][1] + O[1]);

    {
      double	x, y;
      int	i;
      
      for(i = 1; i < 4; i++)
		{
		  x     	= a * (bounds[i][0] + O[0]) + b * (bounds[i][1] + O[1]);
		  y		= c * (bounds[i][0] + O[0]) + d * (bounds[i][1] + O[1]);
		  x_min 	= MIN(x_min, x);
		  x_max	= MAX(x_max, x);
		  y_min	= MIN(y_min, y);
		  y_max	= MAX(y_max, y);
		}

      x_start	= (int) x_min;
      x_end	= (int) x_max;
      y_start	= (int) y_min;
      y_end	= (int) y_max;
    }
   
    //! Create image to be returned
    //!
    R	= new array<u_char>(y_start, y_end, x_start, x_end);

    //! Compute reverse bounding box
    //!
    {
      double xx, yy;
      
      bounds[0][0] = bounds[2][0] = x_min;
      bounds[0][1] = bounds[1][1] = y_min;
      bounds[1][0] = bounds[3][0] = x_max;
      bounds[2][1] = bounds[3][1] = y_max;

      xx_min = yy_min = X_start;
      xx_max = yy_max = Y_start;

      for(int i = 0; i < 4; i++)
		{
		  xx = ia * bounds[i][0] + ib * bounds[i][1] - O[0];
		  yy = ic * bounds[i][0] + id * bounds[i][1] - O[1]; 

		  if(xx < xx_min)
			xx_min = xx;
		  else 
			if(xx > xx_max)
			  xx_max = xx;

		  if(yy < yy_min)
			yy_min = yy;
		  else 
			if(yy > yy_max)
			  yy_max = yy;
		}
    }


    X_end -= 1;
    Y_end -= 1;

    /*
      vector<int>  Xs(int(xx_min - 1), int(xx_max + 1));
      vector<int>  Ys(int(yy_min - 1), int(yy_max + 1));
      int          *_Xs = Xs.storage();
      int          *_Ys = Ys.storage();

      for(int i = Xs.first_index() ; i <= Xs.last_index(); i++)
      _Xs[i] = MAX(X_start, MIN(i, X_end));
   
      for(int i = Ys.first_index() ; i <= Ys.last_index(); i++)
      _Ys[i] = MAX(Y_start, MIN(i, Y_end));
    */
 
    //! Fill it
    //!
    {
      double	back_x, back_y, t1, t2, t0, a0, a1;
      int	bx, by;
      double	v00, v01, v10, v11;
      u_char    **_R, **_I;
      int       value;

      _I = I.storage();
      _R = (*R).storage();

      for(y = y_start; y <= y_end; y++)
		{
		  a0 = ib * y - O[0];
		  a1 = id * y - O[1]; 

		  for(x = x_start; x <= x_end; x++)
			{
			  back_x	= ia * x + a0;
			  back_y 	= ic * x + a1;
			  bx 	= int(back_x);
			  by 	= int(back_y);

			  /*
				bx = _Xs[bx];
				by = _Ys[by];
			  */

			  if(bx < X_start)
				bx = X_start;
			  else if (bx > X_end) 
				bx = X_end;

			  if(by < Y_start)
				by = Y_start;
			  else if (by > Y_end)
				by = Y_end;

			  //! Read surrounding values of back projection 
			  //!
			  v00	= _I[by][bx];
			  v01	= _I[by][bx+1];
			  v10	= _I[by+1][bx];
			  v11	= _I[by+1][bx+1];
		
			  t0	= (back_x - bx);
			  t1	= v00 + t0 * (v01 - v00);
			  t2	= v10 + t0 * (v11 - v10);

			  value   = int(0.5+t1+(back_y-by)*(t2-t1));
			  _R[y][x]= (value > 255) ? 255 : ((value < 0) ? 0 : value);
			}
		}
    }   
   
   
    DeleteIfTemporary(I);
    DeleteIfTemporary(M);
    DeleteIfTemporary(O);
   
    return((*R).temporary());
  }

  //! Return the affine transform of I according to M and 
  //! translation O
  //!
  array<float>&	Affine(array<float>& I, array<double>& M, vector<double> &O)
  {
    array<float> *R;
    int		 cols, rows;
    int		 X_start, X_end;
    int		 Y_start, Y_end;
    int		 x_start, x_end, x;
    int		 y_start, y_end, y;
    double	 a,  b,  c,  d;
    double	 ia, ib, ic, id;
    double	 x_min, x_max, y_min, y_max;
    double	 bounds[4][2];
    double	 determinant;
   
   
    //! Read transform data
    //!
    a	= M[0][0];
    b	= M[0][1];
    c	= M[1][0];
    d	= M[1][1];
   
    //! Compute determinant
    //!
    determinant	= a*d - b*c;
   
    //! Check if transform is singular
    //!
    if(determinant == 0)
      {
		cerr << "Singular transform detected by Affine\n";
		exit(-1);
      }
   
    //! Compute inverse transform
    //!
    ia	=   (d / determinant);
    ib	= - (b / determinant);
    ic	= - (c / determinant);
    id	=   (a / determinant);
   

    //! Get bounds of image
    //!
    cols		= I.cols();
    rows		= I.rows();
    X_start      = I.first_col_index();
    X_end        = I.last_col_index();
    Y_start      = I.first_row_index();
    Y_end        = I.last_row_index();
   
    bounds[0][0]	= (double) X_start;
    bounds[0][1]	= (double) Y_start;
    bounds[1][0]	= bounds[0][0] + (double) cols;
    bounds[1][1]	= bounds[0][1]; 
    bounds[2][0]	= bounds[0][0]; 
    bounds[2][1]	= bounds[0][1] + (double) rows;
    bounds[3][0]	= bounds[1][0];
    bounds[3][1]	= bounds[2][1];

    //! Compute new bounding box
    //!
    x_min = x_max	= a * (bounds[0][0] + O[0]) + b * (bounds[0][1] + O[1]);
    y_min = y_max	= c * (bounds[0][0] + O[0]) + d * (bounds[0][1] + O[1]);

    {
      double	x, y;
      int	i;
      
      for(i = 1; i < 4; i++)
		{
		  x     	= a * (bounds[i][0] + O[0]) + b * (bounds[i][1] + O[1]);
		  y		= c * (bounds[i][0] + O[0]) + d * (bounds[i][1] + O[1]);
		  x_min 	= MIN(x_min, x);
		  x_max	= MAX(x_max, x);
		  y_min	= MIN(y_min, y);
		  y_max	= MAX(y_max, y);
		}

      x_start	= (int) x_min;
      x_end	= (int) x_max;
      y_start	= (int) y_min;
      y_end	= (int) y_max;
    }

    //! Create image to be returned
    //!
    R	= new array<float>(y_start, y_end, x_start, x_end);


    //! Fill it
    //!
    {
      double	back_x, back_y, t1, t2, t0, a0, a1;
      int	bx, by;
      double	v00, v01, v10, v11;
      float     **_R, **_I, value;

      _I = I.storage();
      _R = (*R).storage();

      X_end -= 1;
      Y_end -= 1;

      for(y = y_start; y <= y_end; y++)
		{
		  a0 = ib * y - O[0];
		  a1 = id * y - O[1]; 

		  for(x = x_start; x <= x_end; x++)
			{
			  back_x	= ia * x + a0;
			  back_y 	= ic * x + a1;
			  bx 	= int(back_x);
			  by 	= int(back_y);

			  if(bx < X_start)
				bx = X_start;
			  else if (bx > X_end) 
				bx = X_end;

			  if(by < Y_start)
				by = Y_start;
			  else if (by > Y_end)
				by = Y_end;

			  //! Read surrounding values of back projection 
			  //!
			  v00	= _I[by][bx];
			  v01	= _I[by][bx+1];
			  v10	= _I[by+1][bx];
			  v11	= _I[by+1][bx+1];
		
			  t0	= (back_x - bx);
			  t1	= v00 + t0 * (v01 - v00);
			  t2	= v10 + t0 * (v11 - v10);

			  value   = (t1+(back_y-by)*(t2-t1));
			  _R[y][x]= (value > 255.0) ? 255.0 : ((value < 0.0) ? 0.0 : value);
			}
		}
    }   
   
   
    DeleteIfTemporary(I);
    DeleteIfTemporary(M);
    DeleteIfTemporary(O);
   
    return((*R).temporary());
  }


  //! Return the affine transform of I according to M and 
  //! translation O
  //!
  array<double>&	Affine(array<double>& I, array<double>& M, vector<double> &O)
  {
    array<double> *R;
    int		 cols, rows;
    int		 X_start, X_end;
    int		 Y_start, Y_end;
    int		 x_start, x_end, x;
    int		 y_start, y_end, y;
    double	 a,  b,  c,  d;
    double	 ia, ib, ic, id;
    double	 x_min, x_max, y_min, y_max;
    double	 bounds[4][2];
    double	 determinant;
   
   
    //! Read transform data
    //!
    a	= M[0][0];
    b	= M[0][1];
    c	= M[1][0];
    d	= M[1][1];
   
    //! Compute determinant
    //!
    determinant	= a*d - b*c;
   
    //! Check if transform is singular
    //!
    if(determinant == 0)
      {
		cerr << "Singular transform detected by Affine\n";
		exit(-1);
      }
   
    //! Compute inverse transform
    //!
    ia	=   (d / determinant);
    ib	= - (b / determinant);
    ic	= - (c / determinant);
    id	=   (a / determinant);
   

    //! Get bounds of image
    //!
    cols		= I.cols();
    rows		= I.rows();
    X_start      = I.first_col_index();
    X_end        = I.last_col_index();
    Y_start      = I.first_row_index();
    Y_end        = I.last_row_index();
   
    bounds[0][0]	= (double) X_start;
    bounds[0][1]	= (double) Y_start;
    bounds[1][0]	= bounds[0][0] + (double) cols - 1;
    bounds[1][1]	= bounds[0][1]; 
    bounds[2][0]	= bounds[0][0]; 
    bounds[2][1]	= bounds[0][1] + (double) rows - 1;
    bounds[3][0]	= bounds[1][0];
    bounds[3][1]	= bounds[2][1];

    //! Compute new bounding box
    //!
    x_min = x_max	= a * (bounds[0][0] + O[0]) + b * (bounds[0][1] + O[1]);
    y_min = y_max	= c * (bounds[0][0] + O[0]) + d * (bounds[0][1] + O[1]);

    {
      double	x, y;
      int	i;
      
      for(i = 1; i < 4; i++)
		{
		  x     	= a * (bounds[i][0] + O[0]) + b * (bounds[i][1] + O[1]);
		  y		= c * (bounds[i][0] + O[0]) + d * (bounds[i][1] + O[1]);
		  x_min 	= MIN(x_min, x);
		  x_max	= MAX(x_max, x);
		  y_min	= MIN(y_min, y);
		  y_max	= MAX(y_max, y);
		}

      x_start	= (int) Rint(x_min);
      x_end	= (int) Rint(x_max);
      y_start	= (int) Rint(y_min);
      y_end	= (int) Rint(y_max);
    }
   
    //! Create image to be returned
    //!
    R	= new array<double>(y_start, y_end, x_start, x_end);

    //! Fill it
    //!
    {
      double	back_x, back_y, t1, t2, t0;
      int	bx, by;
      double	v00, v01, v10, v11;
      double     **_R, **_I, value;

      _I = I.storage();
      _R = (*R).storage();

      X_end -= 1;
      Y_end -= 1;
      
      for(y = y_start; y <= y_end; y++)
		{
		  for(x = x_start; x <= x_end; x++)
			{
			  back_x	= ia * x + ib * y - O[0];
			  back_y 	= ic * x + id * y - O[1]; 
			  bx 	= int(floor(back_x));
			  by 	= int(floor(back_y));

			  bx      = MAX(X_start, MIN(bx, X_end));
			  by      = MAX(Y_start, MIN(by, Y_end));
		
			  //! Read surrounding values of back projection 
			  //!
			  v00	= _I[by][bx];
			  v01	= _I[by][bx+1];
			  v10	= _I[by+1][bx];
			  v11	= _I[by+1][bx+1];
		
			  t0	= (back_x - bx);
			  t1	= v00 + t0 * (v01 - v00);
			  t2	= v10 + t0 * (v11 - v10);

			  value   = (t1+(back_y-by)*(t2-t1));
			  _R[y][x]= (value > 255.0) ? 255.0 : ((value < 0.0) ? 0.0 : value);
			}
		}
    }   
   
   
    DeleteIfTemporary(I);
    DeleteIfTemporary(M);
    DeleteIfTemporary(O);
   
    return((*R).temporary());
  }
}
