#include <arrayTools/Array.h>

namespace _ARRAYTOOLS_NAMESPACE
{

//! Get array info as stored in the header data file
//!

int statArray(char *filename, int *r0, int *r1, int *c0, int *c1, int *type)
 {
   animal_header	header;
   FILE                 *in = NULL;
   ifstream             s(filename);
   
   if((in = fopen(filename, "rb")) == NULL)
      {
	return(0);
      }
   else
      {
	freadx((int *) &header,
	       sizeof(int),
	       sizeof(animal_header)/sizeof(int), in);   
   
	if((header.MAGIC != lIMAGE) && (header.MAGIC != lARRAY))
	   {
	     fprintf(stderr, "Bad magic number [%d] for array!\n", header.MAGIC);
	     exit(-1);
	   }

	*type = header.type;
	*c0   = header.focusx;
	*c1   = *c0 + header.cols - 1;
	*r0   = header.focusy;
	*r1   = *r0 + header.rows - 1;
   
	s.close();
	return(1);
      }
 }

//! Append array data to end of file
//!

void appendToFile(char *name, array<float> &A)
 {
   int           y, r0, r1;
   int           x, c0, c1;
   int           s0, s1, arrayType;
   int           recordsize = 0;
   int           file_size = 0;
   FILE          *bout = NULL;
   animal_header header;

   if(statArray(name, &s0, &s1, &c0, &c1, &arrayType) != 1)
      {
	//! if cannot stat it, create it. 
	//!
	A.write(name, anFLOAT);
      }
   else
      {
	//! Open file
	//!
	if((bout = fopen(name, "r+b")) == NULL)
	   {
	     fprintf(stderr,
		     "Cannot open %s for appending data!\n",
		     name);
	     exit(-1);
	   }
	else
	   {
	     //! Check if the file was originally empty
	     //!
	     fseek(bout, 0 , SEEK_END);
	     file_size = ftell(bout);
	     if(file_size == 0)
		{
		  //! Prepare an empty header
		  //!
		  header.MAGIC	= lIMAGE;
		  header.rows	= 0;
		  header.cols	= A.cols();
		  header.type	= anFLOAT;
		  header.focusx	= c0 = A.first_col_index();
		  header.focusy	= A.first_row_index();
		  header.outside     = 0;
		  header.flag	= 0;
		  
		  c1                 = c0 + A.cols() - 1;
		}
	     else
		{
		  //! Position at 0
		  //!
		  fseek(bout, 0 , SEEK_SET);
		  
		  //! Read header
		  //!
		  freadx((int *) &header,
			 sizeof(int),
			 sizeof(animal_header)/sizeof(int), bout);   
		  
		  //! Check data type
		  //!
		  if(header.type != anFLOAT)
		     {
		       fprintf(stderr, "Data type mismatch!\n");
		       exit(-1);
		     }
		  else
		     {
		       //! check record  boundaries
		       //!
		       c0   = header.focusx;
		       c1   = c0 + header.cols - 1;
		       
		       if((c0 != A.first_col_index()) || (c1 != A.last_col_index()))
			  {
			    fprintf(stderr, "Cannot append to file: dimension mismatch!\n");
			    exit(-1);
			  }
		     }
		}
	   }
	
	//! Determine size of row
	//!
	recordsize  = header.cols * sizeof(float);
	
	//! Position at end of file
	//!
	fseek(bout, sizeof(animal_header) + header.rows * recordsize, SEEK_SET);
	
	//! and append data
	//!
	 {
	   float value;
	   
	   r1 = A.last_row_index();
	   r0 = A.first_row_index();
	   
	   for(y = r0; y <= r1; y++)
	     for(x = c0; x <= c1; x++)
		{
		  value	= (float)(A[y][x]);
		  fwritex((char *) &value, sizeof(float), 1, bout);
		}
	 }
	
	//! Position at start of file and 
	//! write updated header
	//!
	header.rows += (r1 - r0 + 1);
	fseek(bout, 0 , SEEK_SET);
	fwritex((char *) &header,
		sizeof(int),
		sizeof(animal_header)/sizeof(int),
		bout);
	
	fclose(bout);
      }

   DeleteIfTemporary(A);
 }


void appendToFile(char *name, array<u_char> &A)
 {
   int           y, r0, r1;
   int           x, c0, c1;
   int           s0, s1, arrayType;
   int           recordsize = 0;
   int           file_size = 0;
   FILE          *bout = NULL;
   animal_header header;
   
   if(statArray(name, &s0, &s1, &c0, &c1, &arrayType) != 1)
      {
	//! if cannot stat it, create it. 
	//!
	A.write(name, anUCHAR);
      }
   else
      {
	//! Open file
	//!
	if((bout = fopen(name, "r+b")) == NULL)
	   {
	     fprintf(stderr,
		     "Cannot open %s for appending data!\n",
		     name);
	     exit(-1);
	   }
	else
	   {
	     //! Check if the file was originally empty
	     //!
	     fseek(bout, 0 , SEEK_END);
	     file_size = ftell(bout);
	     if(file_size == 0)
		{
		  //! Prepare an empty header
		  //!
		  header.MAGIC	= lIMAGE;
		  header.rows	= 0;
		  header.cols	= A.cols();
		  header.type	= anUCHAR;
		  header.focusx	= c0 = A.first_col_index();
		  header.focusy	= A.first_row_index();
		  header.outside     = 0;
		  header.flag	= 0;
		  
		  c1                 = c0 + A.cols() - 1;
		}
	     else
		{
		  //! Position at 0
		  //!
		  fseek(bout, 0 , SEEK_SET);
		  
		  //! Read header
		  //!
		  freadx((int *) &header,
			 sizeof(int),
			 sizeof(animal_header)/sizeof(int), bout);   
		  
		  //! Check data type
		  //!
		  if(header.type != anUCHAR)
		     {
		       fprintf(stderr, "Data type mismatch!\n");
		       exit(-1);
		     }
		  else
		     {
		       //! check record  boundaries
		       //!
		       c0   = header.focusx;
		       c1   = c0 + header.cols - 1;
		       
		       if((c0 != A.first_col_index()) || (c1 != A.last_col_index()))
			  {
			    fprintf(stderr, "Cannot append to file: dimension mismatch!\n");
			    exit(-1);
			  }
		     }
		}
	   }
	
	//! Determine size of row
	//!
	recordsize  = header.cols * sizeof(u_char);
	
	//! Position at end of file
	//!
	fseek(bout, sizeof(animal_header) + header.rows * recordsize, SEEK_SET);
	
	//! and append data
	//!
	 {
	   u_char value;
	   
	   r1 = A.last_row_index();
	   r0 = A.first_row_index();
	   
	   for(y = r0; y <= r1; y++)
	     for(x = c0; x <= c1; x++)
		{
		  value	= (u_char)(A[y][x]);
		  fwritex((char *) &value, sizeof(u_char), 1, bout);
		}
	 }
	
	//! Position at start of file and 
	//! write updated header
	//!
	header.rows += (r1 - r0 + 1);
	fseek(bout, 0 , SEEK_SET);
	fwritex((char *) &header,
		sizeof(int),
		sizeof(animal_header)/sizeof(int),
		bout);
	
	fclose(bout);
      }

   DeleteIfTemporary(A);
 }

}
