#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <arrayTools/aniCommon.h>

extern "C"  {

  SEXP tmBitOr(SEXP ra, 
			   SEXP rb)
  {
	int  na = LENGTH(ra);
	int  nb = LENGTH(rb);
	int *a  = INTEGER(ra);
	int *b  = INTEGER(rb);
	int  N  = (na < nb) ? nb : na;
	SEXP res;
	
	PROTECT(res = allocVector(INTSXP, N));

	int *ires = INTEGER(res);

	for(int i = 0; i < N; i++)
	  ires[i] = a[i%na] | b[i%nb];

	UNPROTECT(1);

	return(res);
  }

  SEXP tmBitAnd(SEXP ra, 
			   SEXP rb)
  {
	int  na = LENGTH(ra);
	int  nb = LENGTH(rb);
	int *a  = INTEGER(ra);
	int *b  = INTEGER(rb);
	int  N  = (na < nb) ? nb : na;
	SEXP res;
	
	PROTECT(res = allocVector(INTSXP, N));

	int *ires = INTEGER(res);

	for(int i = 0; i < N; i++)
	  ires[i] = a[i%na] & b[i%nb];

	UNPROTECT(1);

	return(res);
  }

  SEXP tmBitXor(SEXP ra, 
				SEXP rb)
  {
	int  na = LENGTH(ra);
	int  nb = LENGTH(rb);
	int *a  = INTEGER(ra);
	int *b  = INTEGER(rb);
	int  N  = (na < nb) ? nb : na;
	SEXP res;
	
	PROTECT(res = allocVector(INTSXP, N));

	int *ires = INTEGER(res);

	for(int i = 0; i < N; i++)
	  ires[i] = a[i%na] ^ b[i%nb];

	UNPROTECT(1);

	return(res);
  }

  SEXP tmBitNot(SEXP ra)
  {
	int  n = LENGTH(ra);
	int *a = INTEGER(ra);
	SEXP res;

	
	PROTECT(res = allocVector(INTSXP, n));

	int *ires = INTEGER(res);

	for(int i = 0; i < n; i++)
	  ires[i] = ~a[i];

	UNPROTECT(1);

	return(res);
  }

  SEXP tmBitSum(SEXP ra)
  {
	int  n = LENGTH(ra);
	int *a = INTEGER(ra);
	SEXP res;
	static int mustInit = TRUE;
	static int bits_in_char[256];

	if(mustInit)
	  {
		// Precompute number of bits for 8-bit numbers
		//
		for(int i = 0; i < 256; i++)
		  {
			int count = 0;
			
			int j = i;
			while (j) {
			  count += j & 0x1u;
			  j     >>= 1;
			}
			bits_in_char[i] = count;
		  }

		mustInit = FALSE;
	  }
	
	PROTECT(res = allocVector(INTSXP, n));

	int *ires = INTEGER(res);

	for(int i = 0; i < n; i++)
	  {
		int x = a[i];

		ires[i] =  (bits_in_char [x  & 0xffu] 
					+  bits_in_char [(x >>  8) & 0xffu]
					+  bits_in_char [(x >> 16) & 0xffu]
					+  bits_in_char [(x >> 24) & 0xffu]);
	  }

	UNPROTECT(1);

	return(res);
  }
}
