#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <arrayTools/aniCommon.h>

#define RANK    1
#define CENSUS  2
#define CENSUSM 3
#define IST     4

  void autoFrame(ia::array<int> &N,
				 int            &left,
				 int            &right,
				 int            &up,
				 int            &down)
  {
	int n = N.rows();

	left = right = up = down = 0;

	for(int j = 0; j < n; j++) 
	  {
		if(N[j][0] < left)
		  left  = N[j][0];
		else if (N[j][0] > right)
		  right = N[j][0];
		
		if(N[j][1] < up)
		  up  = N[j][1];
		else if (N[j][1] > down) 
		  down = N[j][1];
	  }

	left = - left;
	up   = - up;
  }

  template<class T> ia::array<int> &ordinalTransform(ia::array<T>   &A,
													 ia::array<int>  N,
													 int             kind)
  {
	int left, right, up, down;
	int x0, x1, y0, y1;
	int n = N.rows();

	autoFrame(N, left, right, up, down);

	A.stat(x0, x1, y0, y1);

	ia::array<int> *R = new ia::array<int>(y0, y1, x0, x1);

	x0 += left;
	x1 -= right;
	y0 += up;
	y1 -= down;

	for(int y = y0; y <= y1; y++)
	  for(int x = x0; x <= x1; x++) 
		{
		  switch(kind) {
		  case RANK:
			{
			  int r = 0;
			  T   v = A[y][x];

			  for(int i = 0; i < n; i++)
				{
				  if(A[y+N[i][1]][x+N[i][0]] < v)
					r++;
				}

			  (*R)[y][x] = r;
			}
			break;
		  case CENSUS:
			{
			  int r = 0;
			  T   v = A[y][x];

			  for(int i = 0; i < n; i++)
				{
				  if(A[y+N[i][1]][x+N[i][0]] < v)
					r = (r|1) << 1;
				  else
					r = r << 1;
				}

			  (*R)[y][x] = r;
			}
			break;
		  case CENSUSM:
			{
			  int r = 0;
			  T   v = 0;

			  for(int i = 0; i < n; i++)
				v += A[y+N[i][1]][x+N[i][0]];
			  v /= n;

			  for(int i = 0; i < n; i++)
				{
				  if(A[y+N[i][1]][x+N[i][0]] < v)
					r = (r|1) << 1;
				  else
					r = r << 1;
				}
			  
			  (*R)[y][x] = r;
			}
			break;
		  case IST:
			{
			  if(A[y+N[0][1]][x+N[0][0]] < A[y][x])
				(*R)[y][x] = 0;
			  else
				(*R)[y][x] = 1;
			}
			break;
		  }
		}

	DeleteIfTemporary(A);
	DeleteIfTemporary(N);
	return((*R).temporary());
  }

extern "C"  {
  SEXP tmOrdinalTransform(SEXP animage_1,   // Must be an animage object
						  SEXP neighborhood,// must be an integer array
						  SEXP mode)        // must be an integer 'type' switch
  {
	// Get info on the neighborhood
	//
	int  wN   = INTEGER(GET_DIM(neighborhood))[1]; // should be 2!
	int  hN   = INTEGER(GET_DIM(neighborhood))[0]; // number of points
	int *nptr = INTEGER(neighborhood);
	int  kind = -1;

	ia::array<int> N(hN, wN);

	for(int i = 0; i < wN; i++)
	  for(int j = 0; j < hN; j++)
		N[j][i] = *nptr++;

	ia::array<int> R;

	const char *transformType = CHAR(STRING_ELT(mode, 0));

	if(strcmp(transformType, "rank") == 0)
	  kind = RANK;
	else if(strcmp(transformType, "census") == 0)
	  kind = CENSUS;
	else if(strcmp(transformType, "censusm") == 0)
	  kind = CENSUSM;
	else
	  kind = IST;

	if(TYPEOF(GET_SLOT(animage_1, install("data"))) == INTSXP) 
	  {
		ia::array<int> A = r2ia_ii(animage_1);
	
		R = ordinalTransform(A, N, kind);
	
		return(ia2r_ii(R));
	  } else if (TYPEOF(GET_SLOT(animage_1, install("data"))) == REALSXP) {
		ia::array<double> A = r2ia_rd(animage_1);
	
		R = ordinalTransform(A, N, kind);
	
		return(ia2r_ii(R));
	  } else {
		return(R_NilValue);
	  }
  }
}
