#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <deque>

#include <arrayTools/aniCommon.h>

template<class T> void MaxMinFilterOld(ia::array<T> &A,
									   int           w,
									   ia::array<T> &m,
									   ia::array<T> &M)
{
  ia::array<T>      a = A;
  int               x0, x1, y0, y1;
  
  a.stat(x0, x1, y0, y1);

  int  cs = x1-x0+1;
  int  rs = y1-y0+1;

  m = a;
  M = m;
  
  a =  a.translate_at(0,0);
  m =  m.translate_at(0,0);
  M =  M.translate_at(0,0);
  
  // We exploit the separable nature of the filter:
  //    first along rows:
  //
  for(int y = 0; y < rs; y++)
	{
	  std::deque<int>   U, L;

	  for(int i = 1; i < cs; ++i)
		{
		  if(i >= w) 
			{
			  M[y][i-w] = a[y][U.size()>0 ? U.front() : i-1];
			  m[y][i-w] = a[y][L.size()>0 ? L.front() : i-1];
			}

		  if(a[y][i] > a[y][i-1] )
			{
			  L.push_back(i-1);
			  
			  if(i == w+L.front())
				L.pop_front();
			  
			  while(U.size() > 0) 
				{
				  if(a[y][i] <= a[y][U.back()]) 
					{
					  if(i == w+U.front())
						U.pop_front();
					  break;
					}
				  U.pop_back();
				}
			} 
		  else 
			{
			  U.push_back(i-1);
			  
			  if(i == w+U.front())
				U.pop_front();
			  
			  while(L.size() > 0)
				{
				  if(a[y][i]>= a[y][L.back()])
					{
					  if(i == w+L.front())
						L.pop_front();
					  break;
					}
				  L.pop_back();
				}
			}
		}

	  M[y][cs-w] = a[y][U.size()>0 ? U.front() : cs-1];
	  m[y][cs-w] = a[y][L.size()>0 ? L.front() : cs-1];
	}

  // then along cols:
  //
  ia::array<T>      am = m;
  ia::array<T>      aM = M;
  
  for(int x = 0; x < cs; x++)
	{
	  std::deque<int>   U, L;

	  for(int i = 1; i < rs; ++i)
		{
		  if(i >= w) 
			{
			  M[i-w][x] = aM[U.size()>0 ? U.front() : i-1][x];
			  m[i-w][x] = am[L.size()>0 ? L.front() : i-1][x];
			}

		  if(a[i][x] > a[i-1][x])
			{
			  L.push_back(i-1);
			  
			  if(i == w+L.front())
				L.pop_front();
			  
			  while(U.size() > 0) 
				{
				  if(a[i][x] <= a[U.back()][x]) 
					{
					  if(i == w+U.front())
						U.pop_front();
					  break;
					}
				  U.pop_back();
				}
			} 
		  else 
			{
			  U.push_back(i-1);
			  
			  if(i == w+U.front())
				U.pop_front();
			  
			  while(L.size() > 0)
				{
				  if(a[i][x] >= a[L.back()][x])
					{
					  if(i == w+L.front())
						L.pop_front();
					  break;
					}
				  L.pop_back();
				}
			}
		}

	  M[rs-w][x] = a[U.size()>0 ? U.front() : rs-1][x];
	  m[rs-w][x] = a[L.size()>0 ? L.front() : rs-1][x];
	}

  M = M.translate_at(y0,x0);
  m = m.translate_at(y0,x0);
}

template<class T> void MaxMinFilter(ia::array<T> &A,
									int           w,
									ia::array<T> &m,
									ia::array<T> &M,
									ia::array<T> &m2,
									ia::array<T> &M2)

{
  ia::array<T>      a = A;
  int               x0, x1, y0, y1;
  
  a.stat(x0, x1, y0, y1);

  int  cs = x1-x0+1;
  int  rs = y1-y0+1;

  m = a;
  M = m;
  
  a =  a.translate_at(0,0);
  m =  m.translate_at(0,0);
  M =  M.translate_at(0,0);
  
  // We exploit the separable nature of the filter:
  //    first along rows:
  //
  for(int y = 0; y < rs; y++)
	{
	  std::deque<int>   U, L;

	  for(int i = 1; i < cs; ++i)
		{
		  if(i >= w) 
			{
			  M[y][i-w] = a[y][U.size()>0 ? U.front() : i-1];
			  m[y][i-w] = a[y][L.size()>0 ? L.front() : i-1];
			}

		  if(a[y][i] > a[y][i-1] )
			{
			  L.push_back(i-1);
			  
			  if(i == w+L.front())
				L.pop_front();
			  
			  while(U.size() > 0) 
				{
				  if(a[y][i] <= a[y][U.back()]) 
					{
					  if(i == w+U.front())
						U.pop_front();
					  break;
					}
				  U.pop_back();
				}
			} 
		  else 
			{
			  U.push_back(i-1);
			  
			  if(i == w+U.front())
				U.pop_front();
			  
			  while(L.size() > 0)
				{
				  if(a[y][i]>= a[y][L.back()])
					{
					  if(i == w+L.front())
						L.pop_front();
					  break;
					}
				  L.pop_back();
				}
			}
		}

	  M[y][cs-w] = a[y][U.size()>0 ? U.front() : cs-1];
	  m[y][cs-w] = a[y][L.size()>0 ? L.front() : cs-1];
	}

  // then along cols: we must now decouple max/min operations
  //
  ia::array<T>      am = m;
  ia::array<T>      aM = M;

  m2 = m;
  M2 = M;

  for(int x = 0; x < cs; x++)
	{
	  std::deque<int>   U, L;

	  for(int i = 1; i < rs; ++i)
		{
		  if(i >= w) 
			{
			  M[i-w][x] = aM[U.size()>0 ? U.front() : i-1][x];
			  m[i-w][x] = am[L.size()>0 ? L.front() : i-1][x];
			}

		  if(aM[i][x] > aM[i-1][x])
			{
			  U.pop_back();

			  while((U.size() > 0)  && (aM[i][x] >  aM[U.back()][x]))
				U.pop_back();
			}
		  
		  if(am[i][x] < am[i-1][x])
			{
			  L.pop_back();
			  
			  while((L.size() > 0) && (am[i][x] < am[L.back()][x]))
				L.pop_back();
			}

		  U.push_back(i);
		  L.push_back(i);

		  if(i == w+U.front())
			U.pop_front();

		  if(i == w+L.front())
			L.pop_front();
		}

	  M[rs-w][x] = aM[U.size()>0 ? U.front() : rs-1][x];
	  m[rs-w][x] = am[L.size()>0 ? L.front() : rs-1][x];
	}

  M = M.translate_at(y0,x0);
  m = m.translate_at(y0,x0);
}


extern "C"  {

  SEXP tmMaxMinFilter(SEXP animage_1, // Must be an animage object
					  SEXP r_window)  // must be integer
  {
	SEXP              ani_r;

	// Get info on the kernel1
	//
	int w    = (INTEGER(r_window))[0];

	PROTECT(ani_r = NEW_LIST(4));

	if(TYPEOF(GET_SLOT(animage_1, install("data"))) == INTSXP) 
	  {
		ia::array<int> A = r2ia_ii(animage_1), Bm, BM, Bm2, BM2;
		
		MaxMinFilter(A, w, Bm, BM, Bm2, BM2);
	
		SET_ELEMENT(ani_r, 0, ia2r_ii(Bm));
		SET_ELEMENT(ani_r, 1, ia2r_ii(BM));
		SET_ELEMENT(ani_r, 2, ia2r_ii(Bm2));
		SET_ELEMENT(ani_r, 3, ia2r_ii(BM2));

	  } else if (TYPEOF(GET_SLOT(animage_1, install("data"))) == REALSXP) {
		ia::array<double> A = r2ia_rd(animage_1), Bm, BM, Bm2, BM2;
	
		MaxMinFilter(A, w, Bm, BM, Bm2, BM2);
	
		SET_ELEMENT(ani_r, 0, ia2r_dr(Bm));
		SET_ELEMENT(ani_r, 1, ia2r_dr(BM));
		SET_ELEMENT(ani_r, 2, ia2r_dr(Bm2));
		SET_ELEMENT(ani_r, 3, ia2r_dr(BM2));

	  } else {
		return(R_NilValue);
	  }
	
	UNPROTECT(1);
   
	return(ani_r);
  }
}
